// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "scene/Scene.hpp"
#include "prefabs/SimpleSpinningCubePrefab.hpp"
#include "scene/components/display/Camera.hpp"
#include "scene/components/ui/text/UIRichTextLabel.hpp"

namespace Dawn {
  class HelloWorldScene : public Scene {
    protected:
      Camera *camera;
      SimpleSpinningCubePrefab *cube;

      void stage() override {
        camera = Camera::create(this);
        camera->transform->lookAt(glm::vec3(3, 3, 3), glm::vec3(0, 0, 0));

        cube = SimpleSpinningCubePrefab::create(this);

        auto canvasItem = this->createSceneItem();
        auto canvas = canvasItem->addComponent<UICanvas>();
        canvas->camera = camera;

        auto newLabelItem = this->createSceneItem();
        auto newLabel = newLabelItem->addComponent<UIRichTextLabel>();
        newLabel->alignment = glm::vec4(0, 0, 0, 0);
        newLabel->alignX = UI_COMPONENT_ALIGN_STRETCH;
        newLabel->alignY = UI_COMPONENT_ALIGN_STRETCH;

        // newLabel->maxWidth = 300.0f;
        newLabel->richText = std::string(
          "<font font=\"font_arial\" size=\"32\" color=\"COLOR_BLUE\">"
            "Hello World\nHow are you?"
            // "Lorem ipsum dolor sit amet, consectetur adipiscing elit. "
            // "Lorem ipsum dolor sit amet, consectetur adipiscing elit. "
            // "Lorem ipsum dolor sit amet, consectetur adipiscing elit. "
            // "Lorem ipsum dolor sit amet, consectetur adipiscing elit. "
            // "Lorem ipsum dolor sit amet, consectetur adipiscing elit. "
            // "Lorem ipsum dolor sit amet, consectetur adipiscing elit. "
            // "Lorem ipsum dolor sit amet, consectetur adipiscing elit. "
            // "Lorem ipsum dolor sit amet, consectetur adipiscing elit. "
            // "Lorem ipsum dolor sit amet, consectetur adipiscing elit. "
            // "Lorem ipsum dolor sit amet, consectetur adipiscing elit. "
            // "Lorem ipsum dolor sit amet, consectetur adipiscing elit. "
            // "Lorem ipsum dolor sit amet, consectetur adipiscing elit. "
          "</font>"
        );
        newLabelItem->transform.setParent(canvas->transform);
      }
      
      std::vector<Asset*> getRequiredAssets() override {
        auto assMan = &this->game->assetManager;
        std::vector<Asset*> assets;
        vectorAppend(&assets, SimpleSpinningCubePrefab::getRequiredAssets(assMan));
        assets.push_back(assMan->get<NewTrueTypeAsset>("font_arial"));
        return assets;
      }

    public:
      HelloWorldScene(DawnGame *game) : Scene(game) {}
  };
}